@@ -1,4 +1,4 @@
-/* Portions Copyright (C) 2009-2019 Greenbone Networks GmbH
+/* Portions Copyright (C) 2009-2020 Greenbone Networks GmbH
  * Based on work Copyright (C) 1998 - 2002 Renaud Deraison
  *               SSL Support Copyright (C) 2001 Michel Arboi
  *
@@ -709,6 +709,10 @@ socket_negotiate_ssl (int fd, openvas_encaps_t transport,
   if (open_SSL_connection (fp, cert, key, passwd, cafile, hostname) <= 0)
     {
       g_free (hostname);
+      g_free (cert);
+      g_free (key);
+      g_free (passwd);
+      g_free (cafile);
       g_message ("Function socket_negotiate_ssl called from %s: "
                  "SSL/TLS connection failed.",
                  nasl_get_plugin_filename ());
@@ -716,6 +720,10 @@ socket_negotiate_ssl (int fd, openvas_encaps_t transport,
       return -1;
     }
   g_free (hostname);
+  g_free (cert);
+  g_free (key);
+  g_free (passwd);
+  g_free (cafile);
   return fd;
 }
 
@@ -1810,10 +1818,10 @@ open_sock_option (struct script_infos *args, unsigned int port, int type,
   if (!t)
     {
       g_message ("ERROR ! NO ADDRESS ASSOCIATED WITH NAME");
-      return -1;
+      return (-1);
     }
   if (IN6_ARE_ADDR_EQUAL (t, &in6addr_any))
-    return -1;
+    return (-1);
   if (IN6_IS_ADDR_V4MAPPED (t))
     {
       bzero ((void *) &addr, sizeof (addr));
@@ -2083,11 +2091,11 @@ qsort_compar (const void *a, const void *b)
   u_short *aa = (u_short *) a;
   u_short *bb = (u_short *) b;
   if (*aa == 0)
-    return 1;
+    return (1);
   else if (*bb == 0)
-    return -1;
+    return (-1);
   else
-    return *aa - *bb;
+    return (*aa - *bb);
 }
 
 /**
