@@ -1,4 +1,4 @@
-/* Copyright (C) 2009-2019 Greenbone Networks GmbH
+/* Copyright (C) 2009-2020 Greenbone Networks GmbH
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
  *
@@ -31,8 +31,6 @@
 
 struct scan_globals
 {
-  char *network_targets;
-  char *network_scan_status;
   GHashTable *files_translation;
   GHashTable *files_size_translation;
   char *scan_id;
