@@ -94,6 +94,7 @@ static init_func libfuncs[] = {
   {"set_kb_item", set_kb_item},
   {"get_kb_item", get_kb_item},
   {"get_kb_list", get_kb_list},
+  {"get_host_kb_index", get_host_kb_index},
   {"security_message", security_message},
   {"log_message", log_message},
   {"error_message", error_message},
@@ -358,12 +359,9 @@ static init_func libfuncs[] = {
   {"smb_file_group_sid", nasl_smb_file_group_sid},
   {"smb_file_trustee_rights", nasl_smb_file_trustee_rights},
   {"win_cmd_exec", nasl_win_cmd_exec},
-  {"scan_phase", scan_phase},
-  {"network_targets", network_targets},
   {"plugin_run_find_service", plugin_run_find_service},
   {"plugin_run_openvas_tcp_scanner", plugin_run_openvas_tcp_scanner},
   {"plugin_run_synscan", plugin_run_synscan},
-  {"plugin_run_nmap", plugin_run_nmap},
   {"isotime_now", nasl_isotime_now},
   {"isotime_is_valid", nasl_isotime_is_valid},
   {"isotime_scan", nasl_isotime_scan},
@@ -392,7 +390,8 @@ static struct
   {"IPPROTO_TCP", IPPROTO_TCP},
   {"IPPROTO_UDP", IPPROTO_UDP},
   {"IPPROTO_ICMP", IPPROTO_ICMP},
-  {"IPROTO_IP", IPPROTO_IP},
+  {"IPPROTO_ICMPV6", IPPROTO_ICMPV6},
+  {"IPPROTO_IP", IPPROTO_IP},
   {"IPPROTO_IGMP", IPPROTO_IGMP},
   {"ENCAPS_AUTO", OPENVAS_ENCAPS_AUTO},
   {"ENCAPS_IP", OPENVAS_ENCAPS_IP},
