@@ -81,7 +81,7 @@ int n;
   sum = (sum >> 16) + (sum & 0xffff); /* add hi 16 to low 16 */
   sum += (sum >> 16);                 /* add carry */
   answer = (int) ~sum;                /* ones-complement, truncate */
-  return answer;
+  return (answer);
 }
 
 /*--------------[ IP ]--------------------------------------------*/
@@ -164,13 +164,13 @@ get_ip_element (lex_ctxt *lexic)
 
   if (ip == NULL)
     {
-      nasl_perror (lexic, "get_ip_element : no valid 'ip' argument!\n");
+      nasl_perror (lexic, "get_ip_element: no valid 'ip' argument\n");
       return NULL;
     }
 
   if (element == NULL)
     {
-      nasl_perror (lexic, "get_ip_element : no valid 'element' argument!\n");
+      nasl_perror (lexic, "get_ip_element: no valid 'element' argument\n");
       return NULL;
     }
 
@@ -379,31 +379,31 @@ dump_ip_packet (lex_ctxt *lexic)
       else
         {
           printf ("------\n");
-          printf ("\tip_hl : %d\n", ip->ip_hl);
-          printf ("\tip_v  : %d\n", ip->ip_v);
-          printf ("\tip_tos: %d\n", ip->ip_tos);
-          printf ("\tip_len: %d\n", UNFIX (ip->ip_len));
-          printf ("\tip_id : %d\n", ntohs (ip->ip_id));
-          printf ("\tip_off: %d\n", UNFIX (ip->ip_off));
-          printf ("\tip_ttl: %d\n", ip->ip_ttl);
+          printf ("\tip_hl  : %d\n", ip->ip_hl);
+          printf ("\tip_v   : %d\n", ip->ip_v);
+          printf ("\tip_tos : %d\n", ip->ip_tos);
+          printf ("\tip_len : %d\n", UNFIX (ip->ip_len));
+          printf ("\tip_id  : %d\n", ntohs (ip->ip_id));
+          printf ("\tip_off : %d\n", UNFIX (ip->ip_off));
+          printf ("\tip_ttl : %d\n", ip->ip_ttl);
           switch (ip->ip_p)
             {
             case IPPROTO_TCP:
-              printf ("\tip_p  : IPPROTO_TCP (%d)\n", ip->ip_p);
+              printf ("\tip_p   : IPPROTO_TCP (%d)\n", ip->ip_p);
               break;
             case IPPROTO_UDP:
-              printf ("\tip_p  : IPPROTO_UDP (%d)\n", ip->ip_p);
+              printf ("\tip_p   : IPPROTO_UDP (%d)\n", ip->ip_p);
               break;
             case IPPROTO_ICMP:
-              printf ("\tip_p  : IPPROTO_ICMP (%d)\n", ip->ip_p);
+              printf ("\tip_p   : IPPROTO_ICMP (%d)\n", ip->ip_p);
               break;
             default:
-              printf ("\tip_p  : %d\n", ip->ip_p);
+              printf ("\tip_p   : %d\n", ip->ip_p);
               break;
             }
-          printf ("\tip_sum: 0x%x\n", ntohs (ip->ip_sum));
-          printf ("\tip_src: %s\n", inet_ntoa (ip->ip_src));
-          printf ("\tip_dst: %s\n", inet_ntoa (ip->ip_dst));
+          printf ("\tip_sum : 0x%x\n", ntohs (ip->ip_sum));
+          printf ("\tip_src : %s\n", inet_ntoa (ip->ip_src));
+          printf ("\tip_dst : %s\n", inet_ntoa (ip->ip_dst));
           printf ("\n");
         }
     }
@@ -437,7 +437,7 @@ forge_tcp_packet (lex_ctxt *lexic)
   if (ip == NULL)
     {
       nasl_perror (lexic,
-                   "forge_tcp_packet : You must supply the 'ip' argument !");
+                   "forge_tcp_packet: You must supply the 'ip' argument\n");
       return NULL;
     }
 
@@ -526,8 +526,7 @@ get_tcp_element (lex_ctxt *lexic)
 
   if (packet == NULL)
     {
-      nasl_perror (lexic,
-                   "get_tcp_element : Error ! No valid 'tcp' argument !\n");
+      nasl_perror (lexic, "get_tcp_element: No valid 'tcp' argument\n");
       return NULL;
     }
 
@@ -544,8 +543,7 @@ get_tcp_element (lex_ctxt *lexic)
   element = get_str_var_by_name (lexic, "element");
   if (!element)
     {
-      nasl_perror (lexic,
-                   "get_tcp_element : Error ! No valid 'element' argument !\n");
+      nasl_perror (lexic, "get_tcp_element: No valid 'element' argument\n");
       return NULL;
     }
 
@@ -572,14 +570,14 @@ get_tcp_element (lex_ctxt *lexic)
   else if (!strcmp (element, "data"))
     {
       retc = alloc_typed_cell (CONST_DATA);
-      retc->size = UNFIX (ip->ip_len) - ntohl (tcp->th_off) * 4;
+      retc->size = UNFIX (ip->ip_len) - (tcp->th_off + ip->ip_hl) * 4;
       retc->x.str_val = g_malloc0 (retc->size);
-      bcopy (tcp + ntohl (tcp->th_off) * 4, retc->x.str_val, retc->size);
+      bcopy ((char *) tcp + tcp->th_off * 4, retc->x.str_val, retc->size);
       return retc;
     }
   else
     {
-      nasl_perror (lexic, "Unknown tcp field %s\n", element);
+      nasl_perror (lexic, "get_tcp_element: Unknown tcp field %s\n", element);
       return NULL;
     }
 
@@ -603,7 +601,7 @@ set_tcp_elements (lex_ctxt *lexic)
   if (!ip)
     {
       nasl_perror (lexic,
-                   "set_tcp_elements : Invalid value for the argument 'tcp'\n");
+                   "set_tcp_elements: Invalid value for the argument 'tcp'\n");
       return NULL;
     }
 
@@ -752,8 +750,8 @@ dump_tcp_packet (lex_ctxt *lexic)
         printf (" (%d)", tcp->th_flags);
       printf ("\n");
       printf ("\tth_win   : %d\n", ntohs (tcp->th_win));
-      printf ("\tth_sum   : 0x%x\n", tcp->th_sum);
-      printf ("\tth_urp   : %d\n", tcp->th_urp);
+      printf ("\tth_sum   : 0x%x\n", ntohs (tcp->th_sum));
+      printf ("\tth_urp   : %d\n", ntohs (tcp->th_urp));
       printf ("\tData     : ");
       c = (char *) ((char *) tcp + sizeof (struct tcphdr));
       if (UNFIX (ip->ip_len) > (sizeof (struct ip) + sizeof (struct tcphdr)))
@@ -1405,10 +1403,11 @@ nasl_send_packet (lex_ctxt *lexic)
   int to = get_int_var_by_name (lexic, "pcap_timeout", 5);
   char *filter = get_str_var_by_name (lexic, "pcap_filter");
   int dfl_len = get_int_var_by_name (lexic, "length", -1);
-  int i = 1;
+  int opt_on = 1;
   struct script_infos *script_infos = lexic->script_infos;
   struct in6_addr *dstip = plug_get_host_ip (script_infos);
   struct in_addr inaddr;
+  int allow_broadcast = 0;
 
   if (dstip == NULL || (IN6_IS_ADDR_V4MAPPED (dstip) != 1))
     return NULL;
@@ -1416,17 +1415,20 @@ nasl_send_packet (lex_ctxt *lexic)
   soc = socket (AF_INET, SOCK_RAW, IPPROTO_RAW);
   if (soc < 0)
     return NULL;
-  if (setsockopt (soc, IPPROTO_IP, IP_HDRINCL, (char *) &i, sizeof (i)) < 0)
+  if (setsockopt (soc, IPPROTO_IP, IP_HDRINCL, (char *) &opt_on,
+                  sizeof (opt_on))
+      < 0)
     perror ("setsockopt ");
 
   while ((ip = get_str_var_by_num (lexic, vi)) != NULL)
     {
+      allow_broadcast = get_int_var_by_name (lexic, "allow_broadcast", 0);
       int sz = get_var_size_by_num (lexic, vi);
       vi++;
 
       if ((unsigned int) sz < sizeof (struct ip))
         {
-          nasl_perror (lexic, "send_packet(): packet is too short!\n");
+          nasl_perror (lexic, "send_packet: packet is too short\n");
           continue;
         }
 
@@ -1437,7 +1439,18 @@ nasl_send_packet (lex_ctxt *lexic)
       bzero (&sockaddr, sizeof (struct sockaddr_in));
       sockaddr.sin_family = AF_INET;
       sockaddr.sin_addr = sip->ip_dst;
-      if (sockaddr.sin_addr.s_addr != inaddr.s_addr)
+
+      if (allow_broadcast)
+        {
+          if (setsockopt (soc, SOL_SOCKET, SO_BROADCAST, &opt_on,
+                          sizeof (opt_on))
+              < 0)
+            perror ("setsockopt ");
+          if (sockaddr.sin_addr.s_addr != INADDR_BROADCAST)
+            allow_broadcast = 0;
+        }
+
+      if (sockaddr.sin_addr.s_addr != inaddr.s_addr && !allow_broadcast)
         {
           char txt1[64], txt2[64];
           strncpy (txt1, inet_ntoa (sockaddr.sin_addr), sizeof (txt1));
@@ -1509,6 +1522,7 @@ nasl_pcap_next (lex_ctxt *lexic)
   struct ip *ret = NULL;
   struct ip6_hdr *ret6 = NULL;
   char *filter = get_str_var_by_name (lexic, "pcap_filter");
+  pcap_if_t *alldevsp = NULL; /* list of capture devices */
   int timeout = get_int_var_by_name (lexic, "timeout", 5);
   tree_cell *retc;
   int sz;
@@ -1536,13 +1550,19 @@ nasl_pcap_next (lex_ctxt *lexic)
           interface = v6_routethrough (dst, &src);
         }
       if (interface == NULL)
-        interface = pcap_lookupdev (errbuf);
+        {
+          if (pcap_findalldevs (&alldevsp, errbuf) < 0)
+            g_message ("Error for pcap_findalldevs(): %s", errbuf);
+          if (alldevsp != NULL)
+            interface = alldevsp->name;
+        }
     }
 
   if (interface != NULL)
     {
       bpf = bpf_open_live (interface, filter);
     }
+
   if (bpf < 0)
     {
       nasl_perror (lexic, "pcap_next: Could not get a bpf\n");
@@ -1627,6 +1647,9 @@ nasl_pcap_next (lex_ctxt *lexic)
     retc->x.str_val = (char *) ret6;
   retc->size = sz;
 
+  if (alldevsp != NULL)
+    pcap_freealldevs (alldevsp);
+
   return retc;
 }
 
@@ -1640,6 +1663,7 @@ nasl_send_capture (lex_ctxt *lexic)
   struct ip *ret = NULL;
   struct ip6_hdr *ret6 = NULL;
   char *filter = get_str_var_by_name (lexic, "pcap_filter");
+  pcap_if_t *alldevsp = NULL; /* list of capture devices */
   int timeout = get_int_var_by_name (lexic, "timeout", 5);
   tree_cell *retc;
   int sz;
@@ -1666,7 +1690,12 @@ nasl_send_capture (lex_ctxt *lexic)
           interface = v6_routethrough (dst, &src);
         }
       if (interface == NULL)
-        interface = pcap_lookupdev (errbuf);
+        {
+          if (pcap_findalldevs (&alldevsp, errbuf) < 0)
+            g_message ("Error for pcap_findalldevs(): %s", errbuf);
+          if (alldevsp != NULL)
+            interface = alldevsp->name;
+        }
     }
 
   if (interface != NULL)
@@ -1755,5 +1784,8 @@ nasl_send_capture (lex_ctxt *lexic)
     retc->x.str_val = (char *) ret6;
   retc->size = sz;
 
+  if (alldevsp != NULL)
+    pcap_freealldevs (alldevsp);
+
   return retc;
 }
