@@ -104,7 +104,7 @@ int n;
   sum = (sum >> 16) + (sum & 0xffff); /* add hi 16 to low 16 */
   sum += (sum >> 16);                 /* add carry */
   answer = (int) ~sum;                /* ones-complement, truncate */
-  return answer;
+  return (answer);
 }
 
 /*--------------[ IP ]--------------------------------------------*/
@@ -147,7 +147,7 @@ forge_ipv6_packet (lex_ctxt *lexic)
   tc = get_int_var_by_name (lexic, "ip6_tc", 0);
   fl = get_int_var_by_name (lexic, "ip6_fl", 0);
 
-  pkt->ip6_ctlun.ip6_un1.ip6_un1_flow = version | tc | fl;
+  pkt->ip6_flow = htonl (version << 28 | tc << 20 | fl);
 
   pkt->ip6_plen = FIX (data_len); /* No extension headers ? */
   pkt->ip6_nxt = get_int_var_by_name (lexic, "ip6_p", 0);
@@ -196,13 +196,13 @@ get_ipv6_element (lex_ctxt *lexic)
 
   if (ip6 == NULL)
     {
-      nasl_perror (lexic, "get_ipv6_element : no valid 'ip' argument!\n");
+      nasl_perror (lexic, "get_ipv6_element: no valid 'ipv6' argument\n");
       return NULL;
     }
 
   if (element == NULL)
     {
-      nasl_perror (lexic, "get_ipv6_element : no valid 'element' argument!\n");
+      nasl_perror (lexic, "get_ipv6_element: no valid 'element' argument\n");
       return NULL;
     }
 
@@ -286,7 +286,7 @@ set_ipv6_elements (lex_ctxt *lexic)
 
   if (o_pkt == NULL)
     {
-      nasl_perror (lexic, "set_ip_elements: missing <ip> field\n");
+      nasl_perror (lexic, "set_ipv6_elements: missing <ip6> field\n");
       return NULL;
     }
 
@@ -330,13 +330,12 @@ dump_ipv6_packet (lex_ctxt *lexic)
       else
         {
           printf ("------\n");
-          printf ("\tip6_v  : %d\n", ip6->ip6_flow >> 28);
-          printf ("\tip6_tc: %d\n", (ip6->ip6_flow >> 20) & 0xff);
-          printf ("\tip6_fl: %d\n", (ip6->ip6_flow) & 0x3ffff);
-          printf ("\tip6_plen: %d\n", UNFIX (ip6->ip6_plen));
-          printf ("\tip6_nxt : %d\n", ntohs (ip6->ip6_nxt));
-          printf ("\tip6_hlim : %d\n", ntohs (ip6->ip6_hlim));
-          switch (ip6->ip6_ctlun.ip6_un1.ip6_un1_nxt)
+          printf ("\tip6_v    : %d\n", ntohl (ip6->ip6_flow) >> 28);
+          printf ("\tip6_tc   : %d\n", (ntohl (ip6->ip6_flow) >> 20) & 0xff);
+          printf ("\tip6_fl   : %d\n", ntohl (ip6->ip6_flow) & 0x3ffff);
+          printf ("\tip6_plen : %d\n", UNFIX (ip6->ip6_plen));
+          printf ("\tip6_hlim : %d\n", ip6->ip6_hlim);
+          switch (ip6->ip6_nxt)
             {
             case IPPROTO_TCP:
               printf ("\tip6_nxt  : IPPROTO_TCP (%d)\n", ip6->ip6_nxt);
@@ -344,16 +343,16 @@ dump_ipv6_packet (lex_ctxt *lexic)
             case IPPROTO_UDP:
               printf ("\tip6_nxt  : IPPROTO_UDP (%d)\n", ip6->ip6_nxt);
               break;
-            case IPPROTO_ICMP:
-              printf ("\tip6_nxt  : IPPROTO_ICMP (%d)\n", ip6->ip6_nxt);
+            case IPPROTO_ICMPV6:
+              printf ("\tip6_nxt  : IPPROTO_ICMPV6 (%d)\n", ip6->ip6_nxt);
               break;
             default:
               printf ("\tip6_nxt  : %d\n", ip6->ip6_nxt);
               break;
             }
-          printf ("\tip6_src: %s\n",
+          printf ("\tip6_src  : %s\n",
                   inet_ntop (AF_INET6, &ip6->ip6_src, addr, sizeof (addr)));
-          printf ("\tip6_dst: %s\n",
+          printf ("\tip6_dst  : %s\n",
                   inet_ntop (AF_INET6, &ip6->ip6_dst, addr, sizeof (addr)));
           printf ("\n");
         }
@@ -461,7 +460,7 @@ forge_tcp_v6_packet (lex_ctxt *lexic)
   if (ip6 == NULL)
     {
       nasl_perror (lexic,
-                   "forge_tcp_packet : You must supply the 'ip' argument !");
+                   "forge_tcp_v6_packet: You must supply the 'ip6' argument\n");
       return NULL;
     }
 
@@ -547,8 +546,7 @@ get_tcp_v6_element (lex_ctxt *lexic)
 
   if (packet == NULL)
     {
-      nasl_perror (lexic,
-                   "get_tcp_element : Error ! No valid 'tcp' argument !\n");
+      nasl_perror (lexic, "get_tcp_v6_element: No valid 'tcp' argument\n");
       return NULL;
     }
 
@@ -563,8 +561,7 @@ get_tcp_v6_element (lex_ctxt *lexic)
   element = get_str_var_by_name (lexic, "element");
   if (!element)
     {
-      nasl_perror (lexic,
-                   "get_tcp_element : Error ! No valid 'element' argument !\n");
+      nasl_perror (lexic, "get_tcp_v6_element: No valid 'element' argument\n");
       return NULL;
     }
 
@@ -592,9 +589,11 @@ get_tcp_v6_element (lex_ctxt *lexic)
     {
       retc = alloc_typed_cell (CONST_DATA);
       retc->size = UNFIX (ip6->ip6_plen) - tcp->th_off * 4;
-      if (retc->size <= 0 || retc->size > ipsz - 40 - tcp->th_off * 4)
+      if (retc->size < 0 || retc->size > ipsz - 40 - tcp->th_off * 4)
         {
-          nasl_perror (lexic, "Erroneous tcp header offset %d", retc->size);
+          nasl_perror (lexic,
+                       "get_tcp_v6_element: Erroneous tcp header offset %d\n",
+                       retc->size);
           deref_cell (retc);
           return NULL;
         }
@@ -604,7 +603,8 @@ get_tcp_v6_element (lex_ctxt *lexic)
     }
   else
     {
-      nasl_perror (lexic, "Unknown tcp field %s\n", element);
+      nasl_perror (lexic, "get_tcp_v6_element: Unknown tcp field %s\n",
+                   element);
       return NULL;
     }
 
@@ -634,8 +634,8 @@ set_tcp_v6_elements (lex_ctxt *lexic)
 
   if (pkt == NULL)
     {
-      nasl_perror (lexic,
-                   "set_tcp_elements : Invalid value for the argument 'tcp'\n");
+      nasl_perror (
+        lexic, "set_tcp_v6_elements: Invalid value for the argument 'tcp'\n");
       return NULL;
     }
 
@@ -1115,8 +1115,11 @@ struct v6pseudo_icmp_hdr
 {
   struct in6_addr s6addr;
   struct in6_addr d6addr;
-  char proto;
   unsigned short len;
+  unsigned char zero1;
+  unsigned char zero2;
+  unsigned char zero3;
+  char proto;
   struct icmp6_hdr icmpheader;
 };
 
@@ -1287,7 +1290,20 @@ forge_icmp_v6_packet (lex_ctxt *lexic)
           break;
         default:
           {
-            nasl_perror (lexic, "forge_icmp_v6_packet: unknown type\n");
+            if (t < 0 || t > 255)
+              {
+                nasl_perror (lexic, "forge_icmp_v6_packet: illegal type %d\n",
+                             t);
+              }
+            else
+              {
+                if (data != NULL)
+                  bcopy (data, &(p[8]), len);
+                icmp->icmp6_id = get_int_var_by_name (lexic, "icmp_id", 0);
+                icmp->icmp6_seq = get_int_var_by_name (lexic, "icmp_seq", 0);
+                size = ip6_sz + 8 + len;
+                sz = 8;
+              }
           }
         }
 
@@ -1657,8 +1673,9 @@ nasl_send_v6packet (lex_ctxt *lexic)
   int dfl_len = get_int_var_by_name (lexic, "length", -1);
   struct script_infos *script_infos = lexic->script_infos;
   struct in6_addr *dstip = plug_get_host_ip (script_infos);
-  int offset = 1;
+  int opt_on = 1;
   char name[INET6_ADDRSTRLEN];
+  int allow_multicast = 0;
 
   if (dstip == NULL || (IN6_IS_ADDR_V4MAPPED (dstip) == 1))
     return NULL;
@@ -1666,18 +1683,19 @@ nasl_send_v6packet (lex_ctxt *lexic)
   if (soc < 0)
     return NULL;
 
-  if (setsockopt (soc, IPPROTO_IPV6, IP_HDRINCL, (char *) &offset,
-                  sizeof (offset))
+  if (setsockopt (soc, IPPROTO_IPV6, IP_HDRINCL, (char *) &opt_on,
+                  sizeof (opt_on))
       < 0)
     perror ("setsockopt");
   while ((ip = get_str_var_by_num (lexic, vi)) != NULL)
     {
+      allow_multicast = get_int_var_by_name (lexic, "allow_multicast", 0);
       int sz = get_var_size_by_num (lexic, vi);
       vi++;
 
       if ((unsigned int) sz < sizeof (struct ip6_hdr))
         {
-          nasl_perror (lexic, "send_packet(): packet is too short!\n");
+          nasl_perror (lexic, "send_v6packet: packet is too short\n");
           continue;
         }
 
@@ -1688,7 +1706,26 @@ nasl_send_v6packet (lex_ctxt *lexic)
       bzero (&sockaddr, sizeof (struct sockaddr_in6));
       sockaddr.sin6_family = AF_INET6;
       sockaddr.sin6_addr = sip->ip6_dst;
-      if (dstip != NULL && !IN6_ARE_ADDR_EQUAL (&sockaddr.sin6_addr, dstip))
+
+      if (allow_multicast)
+        {
+          struct sockaddr_in6 multicast;
+
+          if (setsockopt (soc, SOL_SOCKET, SO_BROADCAST, &opt_on,
+                          sizeof (opt_on))
+              < 0)
+            perror ("setsockopt ");
+
+          bzero (&multicast, sizeof (struct sockaddr_in6));
+          sockaddr.sin6_family = AF_INET6;
+          inet_pton (AF_INET6, "ff02::1", &(multicast.sin6_addr));
+
+          if (!IN6_ARE_ADDR_EQUAL (&sockaddr.sin6_addr, &multicast.sin6_addr))
+            allow_multicast = 0;
+        }
+
+      if (dstip != NULL && !IN6_ARE_ADDR_EQUAL (&sockaddr.sin6_addr, dstip)
+          && !allow_multicast)
         {
           char txt1[64], txt2[64];
           strncpy (
@@ -1700,7 +1737,7 @@ nasl_send_v6packet (lex_ctxt *lexic)
                    sizeof (txt2));
           txt2[sizeof (txt2) - 1] = '\0';
           nasl_perror (lexic,
-                       "send_packet: malicious or buggy script is trying to "
+                       "send_v6packet: malicious or buggy script is trying to "
                        "send packet to %s instead of designated target %s\n",
                        txt1, txt2);
           if (bpf >= 0)
