@@ -1,4 +1,4 @@
-/* Portions Copyright (C) 2009-2019 Greenbone Networks GmbH
+/* Portions Copyright (C) 2009-2020 Greenbone Networks GmbH
  * Portions Copyright (C) 2006 Software in the Public Interest, Inc.
  * Based on work Copyright (C) 1998 - 2006 Tenable Network Security, Inc.
  *
@@ -31,6 +31,6 @@
 #include <gvm/util/kb.h>
 
 void
-attack_network (struct scan_globals *, kb_t *network_kb);
+attack_network (struct scan_globals *);
 
 #endif
