@@ -1,4 +1,4 @@
-/* Portions Copyright (C) 2009-2019 Greenbone Networks GmbH
+/* Portions Copyright (C) 2009-2020 Greenbone Networks GmbH
  * Portions Copyright (C) 2006 Software in the Public Interest, Inc.
  * Based on work Copyright (C) 1998 - 2006 Tenable Network Security, Inc.
  *
@@ -28,9 +28,11 @@
 
 void (*openvas_signal (int signum, void (*handler) (int))) (int);
 void
-sighand_chld ();
+
+sighand_chld (int sig);
+
 void
-sighand_segv ();
+sighand_segv (int sig);
 
 void
 let_em_die (int pid);
